@@ -1,7 +1,7 @@
 /* pam_misc.c -- This is random stuff */
 
 /*
- * $Id: pam_misc.c,v 1.1.1.1 2000/06/20 22:11:19 agmorgan Exp $
+ * $Id: pam_misc.c,v 1.2 2001/01/22 06:07:29 agmorgan Exp $
  */
 
 #include <stdarg.h>
@@ -177,11 +177,15 @@ int _pam_mkargv(char *s, char ***argv, int *argc)
 
 void _pam_sanitize(pam_handle_t *pamh)
 {
+    int old_caller_is = pamh->caller_is;
+
     /*
      * this is for security. We reset the auth-tokens here.
      */
-    pam_set_item(pamh,PAM_AUTHTOK,NULL);
-    pam_set_item(pamh,PAM_OLDAUTHTOK,NULL);
+    __PAM_TO_MODULE(pamh);
+    pam_set_item(pamh, PAM_AUTHTOK, NULL);
+    pam_set_item(pamh, PAM_OLDAUTHTOK, NULL);
+    pamh->caller_is = old_caller_is;
 }
 
 /*
