@@ -1,7 +1,7 @@
 /*
  * pam_private.h
  *
- * $Id: pam_private.h,v 1.3 2001/01/22 06:07:29 agmorgan Exp $
+ * $Id: pam_private.h,v 1.4 2001/02/05 06:50:41 agmorgan Exp $
  *
  * This is the Linux-PAM Library Private Header. It contains things
  * internal to the Linux-PAM library. Things not needed by either an
@@ -47,6 +47,9 @@ struct handler {
     int must_fail;
     int (*func)(pam_handle_t *pamh, int flags, int argc, char **argv);
     int actions[_PAM_RETURN_VALUES];
+    /* set by authenticate, open_session, chauthtok(1st)
+       consumed by setcred, close_session, chauthtok(2nd) */
+    int cached_retval; int *cached_retval_p;
     int argc;
     char **argv;
     struct handler *next;
