@@ -1,7 +1,7 @@
 /* pam_userdb module */
  
 /*
- * $Id: pam_userdb.c,v 1.2 2000/11/19 23:54:06 agmorgan Exp $
+ * $Id: pam_userdb.c,v 1.4 2000/12/04 15:02:16 baggins Exp $
  * Written by Cristian Gafton <gafton@redhat.com> 1996/09/10
  * See the end of the file for Copyright Information
  */
@@ -20,12 +20,12 @@
 
 #include "pam_userdb.h"
 
-#ifdef HAVE_LIBDB
-# define DB_DBM_HSEARCH    1 /* use the dbm interface */
-# include <db.h>
+#ifdef HAVE_NDBM_H
+# include <ndbm.h>
 #else
-# ifdef HAVE_LIBNDBM
-#  include <ndbm.h>
+# ifdef HAVE_DB_H
+#  define DB_DBM_HSEARCH    1 /* use the dbm interface */
+#  include <db.h>
 # else
 #  error "failed to find a libdb or equivalent"
 # endif
@@ -255,8 +255,8 @@ int pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
 
 /* static module data */
 
-struct pam_module _pam_wheel_modstruct = {
-     MODULE_NAME,
+struct pam_module _pam_userdb_modstruct = {
+     "pam_userdb",
      pam_sm_authenticate,
      pam_sm_setcred,
      NULL,
